#include "pyparservisitor.h"


/***** EXPRESSIONS *****/

static void visitNumber( void* state, object value )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitNumber( value );
}

static void visitName( void* state, identifier value, expr_context_ty context )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitName( value, context );
}

static void visitString( void* state, string value )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitString( value );
}

static void visitCall( void* state, expr_ty func, asdl_seq *args, asdl_seq *keywords, expr_ty starargs, expr_ty kwargs )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitCall( func, args, keywords, starargs, kwargs );
}


/***** STATEMENTS *****/


static void visitPrint( void* state, expr_ty destination, asdl_seq* values, bool newline )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitPrint( destination, values, newline );
}

static void visitReturn( void* state, expr_ty value )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitReturn( value );
}

static void visitAssign( void* state, asdl_seq* targets, expr_ty value )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitAssign( targets, value );
}

static void visitFunction( void* state, identifier name, arguments_ty arguments, asdl_seq* body, asdl_seq* decorators )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitFunction( name, arguments, body, decorators );
}

static void visitExpressionStmt( void* state, expr_ty expression )
{
	PyParserVisitor* visit = (PyParserVisitor*) state;
	visit->visitExpressionStmt( expression );
}


static const struct PyParse_Callbacks callbacks = {
	// Statements
	visitFunction,
	visitReturn,
	visitPrint,
	visitAssign,
	visitExpressionStmt,
	
	// Expressions
	visitName,
	visitNumber,
	visitString,
	visitCall,
};

void PyParserVisitor::visitExpression( expr_ty expression )
{
	PyParse_Expression( this, &callbacks, expression );
}

void PyParserVisitor::visitStatement( stmt_ty statement )
{
	PyParse_Statement( this, &callbacks, statement );
}

void PyParserVisitor::visitStatements( asdl_seq* statements )
{
	PyParse_Statements( this, &callbacks, statements );
}

